/**
 ** Parchis4A.java
 **
 ** Copyright (C) 1992, 1999, 2007, 2011, 2012 Mariano Alvarez Fernandez
 ** [e-mail: malfer at telefonica.net]
 **
 ** This file is part of the Parchis4A program.
 **
 ** Parchis4A is free software: you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation, either version 3 of the License, or
 ** (at your option) any later version.
 **
 ** Parchis4A is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with Parchis4A.  If not, see <http://www.gnu.org/licenses/>.
 **
 **/

package com.fgrim.parchis4a;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.StreamCorruptedException;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.PorterDuff;
import android.graphics.drawable.AnimationDrawable;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

public class Parchis4A extends Activity {

//	private static final String TAG = "Parchis4A";
    public static final String SAVE_FILENAME = "parchis4a_save.bin";
	public static final String SAVE_VERSION = "parchis4a_save_6";
    public static final String SAVEREG_FILENAME = "parchis4a_savereg.bin";
	public static final String SAVEREG_VERSION = "parchis4a_savereg_1";
	
    public static final int DIALOG_ABOUT_ID = 3;
    public static final int DIALOG_HELP_ID = 4;
    public static final int DIALOG_EXIT_ID = 5;
    public static final int DIALOG_GAMESPEED_ID = 7;
    public static final int DIALOG_STATS_ID = 8;
    public static final int DIALOG_MANUALDICE_ID = 10;
    public static final int DIALOG_REPEATGAME_ID = 11;

	public static final int colorj[] = {0xffffff55, 0xff5555ff, 0xffff5555, 0xff00aa00};
	public static final int colorj2[] = {0xffadaa39, 0xff0000ad, 0xffad0000, 0xff007500};

	private static String ICICLE_KEY = "parchis4a-state";

	public static DefPartida mDPactual = null;
	public static DefPartida mDPnuevo = null;
	public static DefPartida mDPfav[] = null;
	public static boolean recuperaFavoritos = true;

	public static boolean enableGlobalStats = true;

	private Resources resG;

	private int autointv = 10;
	private int autointvItem = 0;
	
	private EstadoPartida mEP;

	private ParchisView mParchisView;
	private TextView mTVOnPause;
	private Button mTirador;
	private Button mAuxiliar;
	private Button mBPausa;
	private Button mBNewGame;
	private Button mBSameGame;
	private RelativeLayout mDadoManual;
	private ImageView mIVDistancia;
	private BMDistancia mBMDistancia;
	
//	private boolean restartWDialogs = false;
	
	private int blinkPawn = 0;
	private Carrera mCarrera;
	private boolean enCarrera = false;
	private MarchaPodium mMarchaPodium;
	private boolean enMarchaPodium = false;
	
	private int enPausa = 0;
	private boolean enPausaManual = false;
	private boolean enPausaHandle = false;
	private boolean enPausaDado = false;
	private boolean arrancando = true;
	
	private boolean pendNuevoJuego = false;
	
	private AnimationDrawable dadoAnimation;
	private boolean dadoCurrando = false;

	private int playerOrder = 0;
	private boolean yThrowDice = false;
	private int vdadoManual = 0;
	private boolean fastEnd = true;
	private boolean ultraFastEnd = false;
	private boolean fastRobotEnd = false;
	private boolean autoRepetir = false;
	private boolean alreadyWGS = false;
	
	private boolean showTitle = true;
//	private int ultVerExe = 0;
	
	public static Registro mRegistro = null;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        resG = getResources();
        setContentView(R.layout.main);
        if (mDPnuevo == null) mDPnuevo = new DefPartida(DefPartida.PERSONA, DefPartida.MAQUINA,
        		                  DefPartida.MAQUINA, DefPartida.MAQUINA, 2, 2, 2, 2);
        if (mDPactual == null) mDPactual = new DefPartida(mDPnuevo);
        if (mDPfav == null) {
            mDPfav = new DefPartida[4];
            for (int i=0; i<4; i++) mDPfav[i] = null;
        }
        mParchisView = (ParchisView) findViewById(R.id.parchis);
        mTirador = (Button) findViewById(R.id.btirador);
        mAuxiliar = (Button) findViewById(R.id.bauxiliar);
        mBPausa = (Button) findViewById(R.id.bpausa);
        mBNewGame = (Button) findViewById(R.id.bnewgame);
        mBSameGame = (Button) findViewById(R.id.bsamegame);
        mTVOnPause = (TextView) findViewById(R.id.parchis_onpause);
        mDadoManual = (RelativeLayout) findViewById(R.id.dado_manual);
        mIVDistancia = (ImageView) findViewById(R.id.bmdistancia);
        DisplayMetrics metrics = new DisplayMetrics();
        getWindowManager().getDefaultDisplay().getMetrics(metrics);
        mBMDistancia = new BMDistancia(resG, metrics);
    	mIVDistancia.setImageBitmap(mBMDistancia.bm);
    	if (mRegistro == null) {
    	    mRegistro = new Registro();
    	    loadRegistro();
    	}
        
//        Log.d(TAG, "Fecha " + DateFormat.getDateTimeInstance().format(new Date()));

        getPreferences();

        if (!restoreState(savedInstanceState)) {
        	if (!loadSavedGame()) {
        	    setNewGame();
                startNewGameActivity();
        	}
        }
    }
    
    public void getPreferences() {
        SharedPreferences prefs =
        	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        String boardImage = prefs.getString("BoardImage", "1");
        mParchisView.setBoardImage(boardImage);
        mParchisView.showMunecos = prefs.getBoolean("ShowMunecos", true);
        mParchisView.showDestino = prefs.getBoolean("ShowDestino", true);
        mParchisView.showPlPawns = prefs.getBoolean("ShowPlPawns", false);
        String sPlayerOrder = prefs.getString("PlayerOrder", "0");
        playerOrder = Integer.valueOf(sPlayerOrder);
        yThrowDice = prefs.getBoolean("YThrowDice", false);
        fastEnd = prefs.getBoolean("FastEnd", true);
        ultraFastEnd = prefs.getBoolean("UltraFastEnd", false);
        fastRobotEnd = prefs.getBoolean("FastRobotEnd", false);
        autointvItem = prefs.getInt("AutointvItem", 1);
    	autointv = resG.getIntArray(R.array.dlg_gamespeed_value)[autointvItem];
//    	ultVerExe = prefs.getInt("UltVerExe", 0);
        showTitle = prefs.getBoolean("ShowTitle", true);
        enableGlobalStats = prefs.getBoolean("EnableGlobalStats", true);
//        autoRepetir = prefs.getBoolean("AutoRepetir", false);
        autoRepetir = false;
		mParchisView.setKeepScreenOn(autoRepetir ? true : false);
        mIVDistancia.setVisibility(View.GONE);
        if (showTitle) mIVDistancia.setVisibility(View.VISIBLE);
        if (recuperaFavoritos) {
    	    if (prefs.contains("MDPfavorito"))
    		    mDPfav[0] = new DefPartida(prefs.getString("MDPfavorito", "v9  02121212"));
    	    if (prefs.contains("MDPfavorito2"))
    		    mDPfav[1] = new DefPartida(prefs.getString("MDPfavorito2", "v9  02121212"));
    	    if (prefs.contains("MDPfavorito3"))
    		    mDPfav[2] = new DefPartida(prefs.getString("MDPfavorito3", "v9  02121212"));
    	    if (prefs.contains("MDPfavorito4"))
    		    mDPfav[3] = new DefPartida(prefs.getString("MDPfavorito4", "v9  02121212"));
        }
//        Log.d(TAG, "UltVerExe "+ultVerExe);
//        Log.d(TAG, "getPreferences");
    }
    
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
    	if (resultCode == RESULT_OK)
    		pendNuevoJuego = true;
//    		presetNewGame();
//        Log.d(TAG, "onResult");
    }
    
    @Override
    protected void onStart() {
        super.onStart();
        getPreferences();
        recuperaFavoritos = true;
        if (mRegistro.cambiosPendientes) {
        	saveRegistro();
        }
        if (showTitle) {
            mBMDistancia.draw(mEP);
            mIVDistancia.invalidate();
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (!arrancando) terminaPausa();
//        Log.d(TAG, "onResume "+arrancando+" "+enPausa);
        arrancando = false;
        if (pendNuevoJuego)
        	presetNewGame();
        pendNuevoJuego = false;
    }

    @Override
    protected void onPause() {
        super.onPause();
        iniciaPausa();
//        Log.d(TAG, "onPause "+enPausa);
    }

    @Override
    protected void onStop() {
        super.onStop();
        SharedPreferences prefs =
        	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt("AutointvItem", autointvItem);
        editor.putInt("UltVerExe", 12);
        if (mDPfav[0] != null)
            editor.putString("MDPfavorito", mDPfav[0].toString());
        if (mDPfav[1] != null)
            editor.putString("MDPfavorito2", mDPfav[1].toString());
        if (mDPfav[2] != null)
            editor.putString("MDPfavorito3", mDPfav[2].toString());
        if (mDPfav[3] != null)
            editor.putString("MDPfavorito4", mDPfav[3].toString());
        editor.commit();
   }

    public void iniciaPausa() {
    	enPausa++;
    	if (enPausa == 1 && dadoCurrando) {
    		dadoAnimation.stop();
    		enPausaDado = true;
    	}
    }
    
    public void terminaPausa() {
    	if (enPausa == 0) return;
    	enPausa--;
    	if (enPausaDado && enPausa == 0) {
    		dadoAnimation.start();
    		enPausaDado = false;
    	}
    	if (enPausaHandle && enPausa == 0) {
    		mAutoHandler.sleep(10);
    		enPausaHandle = false;
    	}
    }
    
    @Override
    public void onSaveInstanceState(Bundle outState) {
    	super.onSaveInstanceState(outState);
    	Bundle estado = mEP.saveState();
        estado.putIntArray("n0defpa", mDPactual.toArray());
        estado.putIntArray("n0defpn", mDPnuevo.toArray());
        estado.putBoolean("n0epm", enPausaManual);
        estado.putInt("n0vdm", vdadoManual);
        outState.putBundle(ICICLE_KEY, estado);
   }
    
    public void saveGame() {
        try {
            FileOutputStream fout = openFileOutput(SAVE_FILENAME, 0);
            ObjectOutputStream oout = new ObjectOutputStream(fout);
            
            oout.writeObject(SAVE_VERSION);
            oout.writeObject(mDPactual.toArray());
            oout.writeObject(mDPnuevo.toArray());
            oout.writeBoolean(enPausaManual);
            oout.writeInt(vdadoManual);
            mEP.saveState(oout);
            oout.close();

        } catch (FileNotFoundException e) {
            ;
        } catch (IOException e) {
            ;
        }
    }
    
    public boolean restoreState(Bundle savedInstanceState) {
    	if (savedInstanceState == null) return false;
        Bundle map = savedInstanceState.getBundle(ICICLE_KEY);
        if (map == null) return false;

        blinkPawn = 0;
    	enCarrera = false;
    	enMarchaPodium = false;
    	mAutoHandler.removeMessages(0);

//        mDPactual.set(map.getIntArray("n0defpa"));
//        mDPnuevo.set(map.getIntArray("n0defpn"));
        enPausaManual = map.getBoolean("n0epm");
        vdadoManual = map.getInt("n0vdm");
//    	restartWDialogs = true;
 
        mEP = new EstadoPartida();
        mEP.restoreState(map);    	
        
        restartGame();
        return true;
    }

    public boolean loadSavedGame() {
    	try {
            FileInputStream fin = openFileInput(SAVE_FILENAME);
            ObjectInputStream oin = new ObjectInputStream(fin);
            
            String version = (String)oin.readObject();
            if (!version.equals(SAVE_VERSION)) {
                oin.close();
                deleteFile(SAVE_FILENAME);
                return false;
            }
            blinkPawn = 0;
        	enCarrera = false;
        	enMarchaPodium = false;
        	mAutoHandler.removeMessages(0);

            mDPactual.set((int[])oin.readObject());
            mDPnuevo.set((int[])oin.readObject());
            enPausaManual = oin.readBoolean();
            vdadoManual = oin.readInt();
//        	restartWDialogs = true;
            
            mEP = new EstadoPartida();
            mEP.restoreState(oin);
            
            oin.close();
            deleteFile(SAVE_FILENAME);
            restartGame();
            return true;
            
        } catch (FileNotFoundException e) {
            ;
        } catch (StreamCorruptedException e) {
            ;
        } catch (IOException e) {
            ;
        } catch (ClassNotFoundException e) {
            ;
        }
        return false;
    }
    
    public void restartGame() {
    	if (mEP.accion == EstadoPartida.ACC_PARADADO) {
        	mEP.accion = EstadoPartida.ACC_TIRADADO;
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE) {
            mEP.gjg = new GrupoJugadas();
            mEP.gjg.genera(mEP, mEP.turno, mEP.vdado);
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE ||
        	mEP.accion == EstadoPartida.ACC_PASA) {
    		mTirador.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mAuxiliar.invalidate();
        	pintaVdado(mEP.vdado);
        }
        if (mEP.accion == EstadoPartida.ACC_PARTIDATERMINADA) {
        	alreadyWGS = true;
        }
        mAuxiliar.setVisibility(View.GONE);
        mBNewGame.setVisibility(View.GONE);
        mBSameGame.setVisibility(View.GONE);
        mParchisView.setEstadoPartida(mEP);
        mParchisView.invalidate();
//        Log.d(TAG, "RestoreState "+enPausa);
        if (enPausaManual) {
            mTVOnPause.setVisibility(View.VISIBLE);
        	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.play3, 0);
        	enPausa = 1;
        }
		mBPausa.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    	mBPausa.invalidate();
        preparaSigAccion();
    }

    public void saveRegistro() {
        try {
            FileOutputStream fout = openFileOutput(SAVEREG_FILENAME, 0);
            ObjectOutputStream oout = new ObjectOutputStream(fout);
            
            oout.writeObject(SAVEREG_VERSION);
            mRegistro.save(oout);
            oout.close();
            mRegistro.cambiosPendientes = false;

        } catch (FileNotFoundException e) {
            ;
        } catch (IOException e) {
            ;
        }
    }
    
    public boolean loadRegistro() {
    	try {
            FileInputStream fin = openFileInput(SAVEREG_FILENAME);
            ObjectInputStream oin = new ObjectInputStream(fin);
            
            String version = (String)oin.readObject();
            if (!version.equals(SAVEREG_VERSION)) {
                return false;
            }
            mRegistro.load(oin);
            oin.close();
            return true;
            
        } catch (FileNotFoundException e) {
            ;
        } catch (StreamCorruptedException e) {
            ;
        } catch (IOException e) {
            ;
        } catch (ClassNotFoundException e) {
            ;
        }
        return false;
    }
    
    public void presetNewGame() {
    	if (yThrowDice)
        	showDialog(DIALOG_MANUALDICE_ID);
    	else
    	    setNewGame();
    }
    
    public void setNewGame() {
    	alreadyWGS = false;
    	blinkPawn = 0;
    	enCarrera = false;
    	enMarchaPodium = false;
    	mAutoHandler.removeMessages(0);
    	vdadoManual = 0;
    	mDadoManual.setVisibility(View.GONE);
//    	restartWDialogs = true;

    	mEP = new EstadoPartida();
    	mDPactual.quien_sale = playerOrder;
        mDPactual.fin_rapido = fastEnd;
        mDPactual.fin_ultrarapido = ultraFastEnd;
        mDPactual.finrobot_rapido = fastRobotEnd;
    	mDPactual.dado_manual = yThrowDice;
        mEP.iniciaPartida(mDPactual);
        mAuxiliar.setVisibility(View.GONE);
        mBNewGame.setVisibility(View.GONE);
        mBSameGame.setVisibility(View.GONE);
        mBPausa.setVisibility(View.GONE);
    	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.pause3, 0);
        mTVOnPause.setVisibility(View.INVISIBLE);
        enPausa = 0;
        enPausaManual = false;
        enPausaDado = false;
        enPausaHandle = false;
        mParchisView.setEstadoPartida(mEP);
        mParchisView.invalidate();
        preparaSigAccion();
        if (showTitle) {
            mBMDistancia.draw(mEP);
            mIVDistancia.invalidate();
        }
    }
    
    private AutoHandler mAutoHandler = new AutoHandler();

    class AutoHandler extends Handler {

        @Override
        public void handleMessage(Message msg) {
        	if (enPausa > 0) enPausaHandle = true;
        	else Parchis4A.this.autoUpdate();
        }

        public void sleep(long delayMillis) {
        	this.removeMessages(0);
        	sendMessageDelayed(obtainMessage(0), delayMillis);
        }
    };

    public void preparaSigAccion() {
        Resources res = getBaseContext().getResources();

        if (mEP.tipo_jugador[mEP.turno] == DefPartida.PERSONA) {
	        mBPausa.setVisibility(View.GONE);
    	} else {
	        mBPausa.setVisibility(View.VISIBLE);
    	}
        if (mEP.accion == EstadoPartida.ACC_TIRADADO) {
    		mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.dadox, 0, 0, 0);
    		mTirador.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
    		mAuxiliar.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mAuxiliar.invalidate();
    		mBPausa.getBackground().setColorFilter(colorj[mEP.turno], PorterDuff.Mode.SRC_ATOP);
        	mBPausa.invalidate();
        	if (mEP.dado_manual) {
    		    mTirador.setText("dame dado\ngive me dice");
            	mDadoManual.setVisibility(View.VISIBLE);
        	} else {
         	    if (mEP.tipo_jugador[mEP.turno] == DefPartida.PERSONA) {
    		        mTirador.setText(res.getString(R.string.acc_tira));
        	    } else {
    		        mTirador.setText(res.getString(R.string.acc_tiro));
        		    mAutoHandler.sleep(100*autointv);
        	    }
        	}
            mParchisView.invalidate();
    	} else if (mEP.accion == EstadoPartida.ACC_PARADADO) {
    		mTirador.setText(res.getString(R.string.acc_espera));
    		mAutoHandler.sleep(mEP.dado_manual ? 100 :100*autointv);
    	} else if (mEP.accion == EstadoPartida.ACC_MUEVE) {
        	if (mEP.tipo_jugador[mEP.turno] == DefPartida.PERSONA) {
            	mParchisView.mustCalcPawnJgPos = true;
            	mParchisView.showPawnCruz = true;
                if (mEP.gjg.jg[0].acasa)
                	mTirador.setText(res.getString(R.string.acc_acasa));
                else if (mEP.gjg.jg[0].entra) {
                	if (mEP.jgha_salido[mEP.gjg.jg[0].cjugador])
                	    mTirador.setText(res.getString(R.string.acc_sales));
                	else
            	        mTirador.setText(res.getString(R.string.acc_sales2));
                } else if (mEP.gjg.njugadas > 1) {
                    mAuxiliar.setVisibility(View.VISIBLE);
        		    mTirador.setText(res.getString(R.string.acc_mueve2));
                } else
        		    mTirador.setText(res.getString(R.string.acc_mueve));
                if (mEP.gjg.njugadas > 0) {
        		    enCarrera = true;
        		    mCarrera = null;
                }
        	} else {
                if (mEP.gjg.jg[0].acasa)
                	mTirador.setText(res.getString(R.string.acc_acasam));
                else if (mEP.gjg.jg[0].entra) {
                	if (mEP.jgha_salido[mEP.gjg.jg[0].cjugador])
                	    mTirador.setText(res.getString(R.string.acc_salgo));
                	else
                	    mTirador.setText(res.getString(R.string.acc_salgo2));
                } else
        		    mTirador.setText(res.getString(R.string.acc_muevo));
                if (mEP.gjg.njugadas > 0) {
                	blinkPawn = 1;
                	mParchisView.showPawnCruz = true;
        		    mAutoHandler.sleep(18*autointv);
        		    enCarrera = true;
        		    mCarrera = null;
                } else
        		    mAutoHandler.sleep(250*autointv);
        	}
            mParchisView.invalidate();
    	} else if (mEP.accion == EstadoPartida.ACC_PASA) {
        	if (mEP.tipo_jugador[mEP.turno] == DefPartida.PERSONA) {
        		mTirador.setText(res.getString(R.string.acc_pasas));
        	} else {
        		mTirador.setText(res.getString(R.string.acc_paso));
        		mAutoHandler.sleep(200*autointv);
        	}
    	} else {
	        mBPausa.setVisibility(View.GONE);
    		mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.dadox, 0, 0, 0);
    		mTirador.getBackground().setColorFilter(0xff777777, PorterDuff.Mode.SRC_ATOP);
    		mTirador.setText(res.getString(R.string.acc_fin));
            mBNewGame.setVisibility(View.VISIBLE);
            mBSameGame.setVisibility(View.VISIBLE);
    		if (mParchisView.showMunecos) {
    			mAutoHandler.sleep(12*autointv);
    			enMarchaPodium = true;
        	    mMarchaPodium = null;
    		} else {
        	    Animation mAnimation = AnimationUtils.loadAnimation(this, R.anim.salto);
        	    mParchisView.startAnimation(mAnimation);
    		}
    		if (enableGlobalStats && !alreadyWGS) {
    			if (mRegistro.registra(mEP)) saveRegistro();
    		}
			alreadyWGS = true;
    		if (autoRepetir) {
    			setNewGame();
    		}
    	}
    }

    private void pintaVdado(int vdado) {
		int res;
		if (vdado == 1) res = R.drawable.dado1;
		else if (vdado == 2) res = R.drawable.dado2;
		else if (vdado == 3) res = R.drawable.dado3;
		else if (vdado == 4) res = R.drawable.dado4;
		else if (vdado == 5) res = R.drawable.dado5;
		else if (vdado == 6) res = R.drawable.dado6;
		else if (vdado == 7) res = R.drawable.dado6;
		else if (vdado == 10) res = R.drawable.dado10;
		else if (vdado == 20) res = R.drawable.dado20;
		else return;
		mTirador.setCompoundDrawablesWithIntrinsicBounds(res, 0, 0, 0);
    }
    
    public void ajustaAccion() {
        if (mEP.accion == EstadoPartida.ACC_TIRADADO) {
    	    mTirador.setCompoundDrawablesWithIntrinsicBounds(R.drawable.mueve_dado, 0, 0, 0);
        	Drawable bdraw[] = mTirador.getCompoundDrawables();
        	dadoAnimation = (AnimationDrawable) bdraw[0];
    	    dadoAnimation.start();
    	    dadoCurrando = true;
        }
        if (mEP.accion == EstadoPartida.ACC_PARADADO) {
        	Drawable bdraw[] = mTirador.getCompoundDrawables();
        	dadoAnimation = (AnimationDrawable) bdraw[0];
    	    dadoAnimation.stop();
    	    pintaVdado(mEP.vdado);
    	    dadoCurrando = false;
        }
        if (mEP.accion == EstadoPartida.ACC_MUEVE) {
        	if (mEP.sigvdado >= 10) {
        		pintaVdado(mEP.sigvdado);
        	}
            mAuxiliar.setVisibility(View.GONE);
            mParchisView.invalidate();
            if (showTitle) {
                mBMDistancia.draw(mEP);
                mIVDistancia.invalidate();
            }
        }
    }
    
    public boolean doBlinkPawn() {
    	if (blinkPawn == 0) return false;
      	mParchisView.showPawnCruz = !mParchisView.showPawnCruz;
        mParchisView.invalidate();
        if (blinkPawn > 7)
        	blinkPawn = 0;
        else
        	blinkPawn++;
        mAutoHandler.sleep(18*autointv);
        return true;
    }

    public boolean doEnCarrera() {
    	int intv = 12;
    	
    	if (!enCarrera) return false;
    	if (mCarrera == null)
    		mCarrera = new Carrera(mEP.gjg.jg[mEP.njgelegida]);
    	if (mCarrera == null || mCarrera.nula) {
    		enCarrera = false;
    		return false;
    	}
    	if (mCarrera.sigpaso >= mCarrera.npasos) {
    		if (!mCarrera.especial || mCarrera.cuentaespecial > 5) {
    		    enCarrera = false;
        	    mParchisView.modoCarrera = false;
    		    return false;
    		}
    		intv = (mCarrera.cuentaespecial % 2 == 0) ? 24 : 10;
    	}
    	mParchisView.modoCarrera = true;
    	mParchisView.setCarrera(mCarrera);
    	mParchisView.invalidate();
        mAutoHandler.sleep(intv*autointv);
        return true;
    }

    public boolean doEnMarchaPodium() {
    	if (!enMarchaPodium) return false;
    	if (mMarchaPodium == null) {
    		mMarchaPodium = new MarchaPodium(mEP);
    	    if (mMarchaPodium == null || mMarchaPodium.njg == 0) {
    		    enMarchaPodium = false;
    		    mMarchaPodium = null;
    		    return true;
    	    }
    	} else {
    		mMarchaPodium.avanza();
    	}
    	if (mMarchaPodium.fin) {
    		enMarchaPodium = false;
    		mParchisView.modoMarchaPodium = false;
        	mParchisView.invalidate();
    	    Animation mAnimation = AnimationUtils.loadAnimation(this, R.anim.zoompod);
    	    mParchisView.startAnimation(mAnimation);
    		return true;
    	}
    	mParchisView.modoMarchaPodium = true;
    	mParchisView.setMarchaPodium(mMarchaPodium);
    	mParchisView.invalidate();
        mAutoHandler.sleep(12*autointv);
        return true;
    }
    
    public void autoUpdate() {
    	if (doBlinkPawn()) return;
    	if (doEnCarrera()) return;
    	if (doEnMarchaPodium()) return;
    	mEP.iniciaAccion(vdadoManual);
    	ajustaAccion();
    	mEP.terminaAccion();
    	preparaSigAccion();
    }
    
    public void bTirador(View view) {
        Resources res = getBaseContext().getResources();

        if (enPausa > 0) {
    		Toast.makeText(this, R.string.toast_onpause, Toast.LENGTH_SHORT).show();
        	return;
        }
        if (mEP.accion == EstadoPartida.ACC_PARTIDATERMINADA) {
//    		CharSequence str = res.getString(R.string.toast_gameover);
//    		Toast t = Toast.makeText(this, str, Toast.LENGTH_SHORT);
//    		t.setGravity(Gravity.CENTER, 0, 0);
//    		t.show();;
            if (!enMarchaPodium) {
          	    Animation mAnimation = AnimationUtils.loadAnimation(this, R.anim.vuelta);
        	    mParchisView.startAnimation(mAnimation);
        		if (mParchisView.showMunecos) {
        			mAutoHandler.sleep(12*autointv);
        			enMarchaPodium = true;
            	    mMarchaPodium = null;
        		}
            }
    		return;
    	}
        if (mEP.dado_manual && mEP.accion == EstadoPartida.ACC_TIRADADO) {
        	return;
        }
        if (mEP.tipo_jugador[mEP.turno] != DefPartida.PERSONA) {
    		CharSequence str;
    		if (mEP.num_jgh > 0)
    			str = res.getString(R.string.toast_waitturn);
    		else
    			str = res.getString(R.string.toast_stopauto);
    		Toast.makeText(this, str, Toast.LENGTH_SHORT).show();
    		return;
        }
    	if (mEP.accion == EstadoPartida.ACC_PARADADO) {
    		return;
    	}

        if (doEnCarrera()) return;
    	mEP.iniciaAccion(vdadoManual);
    	ajustaAccion();
    	mEP.terminaAccion();
    	preparaSigAccion();
    }

    public void bAuxiliar(View view) {
        if (enPausa > 0) return;
        mEP.njgelegida++;
        if (mEP.njgelegida >= mEP.gjg.njugadas) mEP.njgelegida = 0;
        mParchisView.invalidate();
    }

    public void bPausa(View view) {
    	if (mBPausa.getVisibility() != View.VISIBLE)
    		return;
        if (mEP.dado_manual && mEP.accion == EstadoPartida.ACC_TIRADADO)
        	return;

        if (enPausa > 0) {
        	terminaPausa();
        	enPausaManual = false;
        	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.pause3, 0);
            mTVOnPause.setVisibility(View.INVISIBLE);
        	}
        else {
        	iniciaPausa();
        	enPausaManual = true;
        	mBPausa.setCompoundDrawablesWithIntrinsicBounds(0, 0, R.drawable.play3, 0);
            mTVOnPause.setVisibility(View.VISIBLE);
        }
    }

    public void bNewGame(View view) {
    	if (mBNewGame.getVisibility() != View.VISIBLE)
    		return;
//    	showDialog(DIALOG_WELCOME_ID);
    	startNewGameActivity();
    }
    
    public void bSameGame(View view) {
    	if (mBSameGame.getVisibility() != View.VISIBLE)
    		return;
    	showDialog(DIALOG_REPEATGAME_ID);
    }
    
    public void pbDadoManual(View view) {
    	int id = view.getId();
    	switch (id) {
    	case R.id.bdado1:
    		vdadoManual = 1;
    		break;
    	case R.id.bdado2:
    		vdadoManual = 2;
    		break;
    	case R.id.bdado3:
    		vdadoManual = 3;
    		break;
    	case R.id.bdado4:
    		vdadoManual = 4;
    		break;
    	case R.id.bdado5:
    		vdadoManual = 5;
    		break;
    	case R.id.bdado6:
    		vdadoManual = 6;
    		break;
    	default:
    		vdadoManual = 0;
    		return;
    	}
    	mDadoManual.setVisibility(View.GONE);
		mAutoHandler.sleep(10);
    }

    public void pbTitulo(View view) {
    	showDialog(DIALOG_STATS_ID);
    }
    
    public void startNewGameActivity() {
    	Intent nuevoJuegoActivity = new Intent(getBaseContext(), NuevoJuego.class);
        startActivityForResult(nuevoJuegoActivity, 0);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.parchis4a_menu, menu);
    	menu.findItem(R.id.menu_settings).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_preferences));
    	menu.findItem(R.id.menu_help).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_help));
    	menu.findItem(R.id.menu_selectplayers).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_set_as));
    	menu.findItem(R.id.menu_stats).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_search));
    	menu.findItem(R.id.menu_fast).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_view));
    	menu.findItem(R.id.menu_reg).setIcon(
            getResources().getDrawable(android.R.drawable.ic_menu_info_details));
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle item selection
        switch (item.getItemId()) {
        case R.id.menu_selectplayers:
        	startNewGameActivity();
        	return true;
        case R.id.menu_stats:
        	showDialog(DIALOG_STATS_ID);
       	    return true;
        case R.id.menu_fast:
        	showDialog(DIALOG_GAMESPEED_ID);
       	    return true;
        case R.id.menu_settings:
        	Intent settingsActivity = new Intent(getBaseContext(), Preferences.class);
            startActivity(settingsActivity);
            return true;
        case R.id.menu_reg:
        	if (enableGlobalStats) {
        	    Intent regActivity = new Intent(getBaseContext(), RegistroAct.class);
                startActivity(regActivity);
        	} else {
                Toast.makeText(this, R.string.toast_noglobalstats, Toast.LENGTH_LONG).show();
        	}
            return true;
        case R.id.menu_help:
        	showDialog(DIALOG_HELP_ID);
            return true;
       default:
            return super.onOptionsItemSelected(item);
        }
    }
    
    @Override
    protected Dialog onCreateDialog(int id) {
        Dialog dialog;

        switch(id) {
        case DIALOG_STATS_ID:
        	LayoutInflater inflaterst = (LayoutInflater) getSystemService(LAYOUT_INFLATER_SERVICE);
        	View layoutst = inflaterst.inflate(R.layout.stats_layout,
                    (ViewGroup) findViewById(R.id.stats_layout_root));
        	AlertDialog.Builder builderst = new AlertDialog.Builder(this);
        	builderst.setTitle(R.string.stats_title);
        	builderst.setView(layoutst);

        	builderst.setPositiveButton(R.string.dlg_ok,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                    	dialog.cancel();
                    }
                });
        	
        	dialog = builderst.create();
        	dialog.setCancelable(true);
            dialog.setCanceledOnTouchOutside(true);

        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
            break;
        case DIALOG_ABOUT_ID:
        	LayoutInflater inflatera = (LayoutInflater) getSystemService(LAYOUT_INFLATER_SERVICE);
        	View layouta = inflatera.inflate(R.layout.about_layout,
                    (ViewGroup) findViewById(R.id.about_layout_root));
        	AlertDialog.Builder buildera = new AlertDialog.Builder(this);
        	buildera.setTitle(R.string.about_title);
        	buildera.setIcon(R.drawable.icon);
        	buildera.setView(layouta);

        	buildera.setPositiveButton(R.string.dlg_ok,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                    	showDialog(DIALOG_HELP_ID);
                    	dialog.cancel();
                    }
                });
        	
        	dialog = buildera.create();
        	dialog.setCancelable(true);
//            dialog.setCanceledOnTouchOutside(true);

        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
            break;
        case DIALOG_HELP_ID:
        	LayoutInflater inflaterh = (LayoutInflater) getSystemService(LAYOUT_INFLATER_SERVICE);
        	View layouth = inflaterh.inflate(R.layout.help_layout,
                    (ViewGroup) findViewById(R.id.help_layout_root));
        	AlertDialog.Builder builderh = new AlertDialog.Builder(this);
        	builderh.setTitle(R.string.help_title);
        	builderh.setIcon(R.drawable.icon);
        	builderh.setView(layouth);

        	builderh.setPositiveButton(R.string.dlg_ok,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                    	dialog.cancel();
                    }
                });
            builderh.setNegativeButton(R.string.help_about,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        showDialog(DIALOG_ABOUT_ID);
       	                dialog.cancel();
                    }
                });
        	
        	dialog = builderh.create();
        	dialog.setCancelable(true);
//            dialog.setCanceledOnTouchOutside(true);

        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
            break;
        case DIALOG_EXIT_ID:
        	AlertDialog.Builder builderde = new AlertDialog.Builder(this);
        	builderde.setMessage(resG.getString(R.string.dlg_exit_title));
        	builderde.setCancelable(false);
        	builderde.setPositiveButton(resG.getString(R.string.dlg_yes),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
        	            if (!mEP.haTerminado()) saveGame();
        	            finish();
        	        }
        	    });
        	builderde.setNegativeButton(resG.getString(R.string.dlg_cancel),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
    	        	    terminaPausa();
   	                    dialog.cancel();
        	        }
        	    });
        	builderde.setNeutralButton(resG.getString(R.string.dlg_no),
    	    	new DialogInterface.OnClickListener() {
    	            public void onClick(DialogInterface dialog, int id) {
        	            finish();
    	            }
    	        });
        	dialog = builderde.create();
        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	break;
        case DIALOG_REPEATGAME_ID:
        	AlertDialog.Builder builderrg = new AlertDialog.Builder(this);
        	builderrg.setMessage(resG.getString(R.string.dlg_repeatgame_title));
        	builderrg.setCancelable(true);
        	builderrg.setPositiveButton(resG.getString(R.string.dlg_yes),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
        	        	presetNewGame();
   	                    dialog.cancel();
        	        }
        	    });
        	builderrg.setNegativeButton(resG.getString(R.string.dlg_cancel),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
   	                    dialog.cancel();
        	        }
        	    });
        	dialog = builderrg.create();
        	break;
        case DIALOG_MANUALDICE_ID:
        	AlertDialog.Builder builderdmd = new AlertDialog.Builder(this);
        	builderdmd.setMessage(resG.getString(R.string.dlg_manualdice_title));
        	builderdmd.setCancelable(false);
        	builderdmd.setPositiveButton(resG.getString(R.string.dlg_yes_md),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
        	        	setNewGame();
	                    dialog.cancel();
        	        }
        	    });
        	builderdmd.setNegativeButton(resG.getString(R.string.dlg_no_md),
        	    new DialogInterface.OnClickListener() {
        	        public void onClick(DialogInterface dialog, int id) {
                        yThrowDice = false;
                        SharedPreferences prefs =
                        	PreferenceManager.getDefaultSharedPreferences(getBaseContext());
                        SharedPreferences.Editor editor = prefs.edit();
                        editor.putBoolean("YThrowDice", yThrowDice);
                        editor.commit();
        	        	setNewGame();
   	                    dialog.cancel();
        	        }
        	    });
        	dialog = builderdmd.create();
        	break;
        case DIALOG_GAMESPEED_ID:
        	AlertDialog.Builder buildergs = new AlertDialog.Builder(this);
        	buildergs.setTitle(resG.getString(R.string.dlg_gamespeed_title));
        	buildergs.setSingleChoiceItems(R.array.dlg_gamespeed_item, autointvItem,
        		new DialogInterface.OnClickListener() {
        	    public void onClick(DialogInterface dialog, int item) {
        	    	autointvItem = item;
        	    	autointv = resG.getIntArray(R.array.dlg_gamespeed_value)[item];
        	    	String s1 = resG.getString(R.string.toast_gamespeed);
        	    	String s2 = resG.getStringArray(R.array.dlg_gamespeed_item)[item];
        	        Toast.makeText(getApplicationContext(), s1+" "+s2, Toast.LENGTH_SHORT).show();
        	    	dialog.cancel();
        	    }
        	});
 	        buildergs.setPositiveButton(resG.getString(R.string.dlg_cancel),
    	    	new DialogInterface.OnClickListener() {
    	        public void onClick(DialogInterface dialog, int id) {
        	    	dialog.cancel();
    	        }
    	    });
        	dialog = buildergs.create();
        	dialog.setOnShowListener(new DialogInterface.OnShowListener() {
        		public void onShow(DialogInterface dialog) {
                	iniciaPausa();
        		}
        	});
        	dialog.setOnDismissListener(new DialogInterface.OnDismissListener() {
        		public void onDismiss(DialogInterface dialog) {
        			terminaPausa();
        		}
        	});
        	break;
        default:
            dialog = null;
        }
        return dialog;
    }

    int ridEstDado[][] = {
    	{R.id.tv0d1, R.id.tv0d2, R.id.tv0d3, R.id.tv0d4, R.id.tv0d5, R.id.tv0d6},
    	{R.id.tv1d1, R.id.tv1d2, R.id.tv1d3, R.id.tv1d4, R.id.tv1d5, R.id.tv1d6},
    	{R.id.tv2d1, R.id.tv2d2, R.id.tv2d3, R.id.tv2d4, R.id.tv2d5, R.id.tv2d6},
    	{R.id.tv3d1, R.id.tv3d2, R.id.tv3d3, R.id.tv3d4, R.id.tv3d5, R.id.tv3d6} };
    int ridEstPasa[]= {R.id.tv0lj1, R.id.tv1lj1, R.id.tv2lj1, R.id.tv3lj1}; 
    int ridEstCome[]= {R.id.tv0lj2, R.id.tv1lj2, R.id.tv2lj2, R.id.tv3lj2}; 
    int ridEstComida[]= {R.id.tv0lj3, R.id.tv1lj3, R.id.tv2lj3, R.id.tv3lj3}; 
    int ridEstACasa[]= {R.id.tv0lj4, R.id.tv1lj4, R.id.tv2lj4, R.id.tv3lj4}; 
    int ridEstPOS[]= {R.id.tv0lj5, R.id.tv1lj5, R.id.tv2lj5, R.id.tv3lj5}; 

    @Override
    protected void onPrepareDialog(int id, Dialog dialog) {
        switch(id) {
        case DIALOG_STATS_ID:
        	TextView mTView;
        	String s;
        	ImageView mIViewst = (ImageView) dialog.findViewById(R.id.bmgroup);
        	Bitmap bmst = BMPlayerGroup.GenerateEP(resG, mEP, false);
        	mIViewst.setImageBitmap(bmst);
//        	AlertDialog adlg = (AlertDialog)dialog;
//        	adlg.setIcon(new BitmapDrawable(resG, bm));
    		for (int c=EstadoPartida.AMAR; c<=EstadoPartida.VERD; c++) {
    			boolean mostrar = mEP.tipo_jugador[c] != DefPartida.NOJUEGA;
    			for (int ind=0; ind<6; ind++) {
                    mTView = (TextView) dialog.findViewById(ridEstDado[c][ind]);
                    s = mostrar ?
                    	String.valueOf( (int)(((mEP.estDado[c][ind] * 100.0) / mEP.estTotDado[c]) + 0.5) ) + "%"
                    	: "-";
                    mTView.setText(s);
    			}
                mTView = (TextView) dialog.findViewById(ridEstPasa[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estPasa[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstCome[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estCome[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstComida[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estComida[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstACasa[c]);
                mTView.setText(mostrar ? String.valueOf(mEP.estACasa[c]) : "-");
                mTView = (TextView) dialog.findViewById(ridEstPOS[c]);
                s = (!mostrar) ? "-" :
                	((mEP.jgha_terminado[c] < 0) ? ((320-mEP.distancia[c])*100/320) + "%" :
                		(mEP.jgha_terminado[c] + 1) + "º");
                mTView.setText(s);
    		}
            break;
        }
    }

    @Override
    public void onBackPressed() {
    	if (mEP.haComenzado() && !mEP.haTerminado())
    	    showDialog(DIALOG_EXIT_ID);
    	else
    	    finish();
    }
}